@@ -14,4 +14,7 @@ test_that("brentMin correctly handles bad inputs", {
   # TODO: check if the interval is missing
   # TODO: there should be an error in expect_error(brentMin(sin, c(0, -1)), "must be strictly less")
   expect_error(brentMin(sin, 1), "must be a vector of length 2")
+  expect_error(brentMin(function(x) -cos(x), c(-0.5, 0.4), tol = -1), "must be > 0 and finite")
+  suppressWarnings(expect_warning(brentMin(function(x) NULL, c(-0.5, 0.4), maxiter = 1), "Function returned NULL"))
+  suppressWarnings(expect_warning(brentMin(function(x) NA, c(-0.5, 0.4), maxiter = 1), "Function returned NA"))
 })
